/**
 * Generator for combinatorial objects of n bits with k 1s,
 * with one settable listener.
 */
public class Generator {
    
    /** A listener, or null */
    private GeneratorListener listener;
    
    /**
     * Sets listener for generator events.
     */
    public void setListener(GeneratorListener listener) {
        this.listener = listener;
    }

    /**
     * @pre   0 <= k <= n
     * @post  each n-bit pattern with  k 1's
     *   has been reported to listener exactly once
     */
    public void generate(int n, int k) {
        generate("", n, k);
    }
    
    /**
     * @pre   0 <= k <= n
     * @post  each bit pattern with prefix s and n additional bits,
     *   of which k 1's, has been reported to listener exactly once
     * @bound n
     */
    void generate(String s, int n, int k) {
        if (n == 0) {  // k == 0, base case
            if (listener != null) {
                listener.objectGenerated(s);
            }
        } else {  // 0 < n, inductive step
            for (int b = 0; b <= 1; ++b) {
                if (0 <= k - b && k - b <= n - 1) {
                    generate(s + b, n - 1, k - b);
                }
            }
        }
    }

}