/**
 * Generator for combinatorial objects of n bits with k 1s,
 * using a callback (listener) parameter.
 */
public class GeneratorPlain {
    
    /**
     * @pre   0 <= k <= n
     * @post  each n-bit pattern with k 1's
     *   has been reported to listener exactly once
     */
    public static void generate(int n, int k, GeneratorListener listener) {
        generate("", n, k, listener);
    }
    
    /**
     * @pre   0 <= k <= n
     * @post  each bit pattern with prefix s and n additional bits,
     *   of which k 1's, has been printed exactly once
     * @bound n
     */
    public static void generate(String s, int n, int k,
            GeneratorListener listener) {
        if (n == 0) {  // k == 0, base case
            if (listener != null) {
                listener.objectGenerated(s);
            }
        } else {  // 0 < n, inductive step
            for (int b = 0; b <= 1; ++b) {
                if (0 <= k - b && k - b <= n - 1) {
                    generate(s + b, n - 1, k - b, listener);
                }
            }
        }
    }
    // N.B. listener is passed down the chain of recursive calls

}