public class Main {
    
    /**
     * Listener that prints the objects generated.
     */
    static class Printer implements GeneratorListener {
        
        public void objectGenerated(String object) {
            System.out.println(object);
        }
    }
    
    /**
     * Listener that counts the objects generated.
     */
    static class Counter implements GeneratorListener {
        private int count = 0; // number of objects generated
        
        public void objectGenerated(String object) {
            ++ count;
        }
        
        public int getCount() {
            return count;
        }
    }
    
    /**
     * Example illustrating how to call generator with
     * listener object as parameter.
     */
    public static void examplePlain(int n, int k) {
        System.out.println("As parameter: n, k == " + n + ", " + k);
        GeneratorPlain.generate("", n, k, new Printer());        
        final Counter counter = new Counter();
        GeneratorPlain.generate("", n, k, counter);
        final int count = counter.getCount();
        System.out.println("# combinations = " + count);
    }
    
    /**
     * Example illustrating how to call generator with
     * listener object from anonymous inner class as parameter.
     */
    public static void exampleAnonymous(int n, int k) {
        System.out.println("Anonymous class: n, k == " + n + ", " + k);

        GeneratorPlain.generate("", n, k, new GeneratorListener() {
            public void objectGenerated(String object) {
                System.out.print(".");
            }
        });
        
        System.out.println();
    }
    
    /**
     * Example illustrating how to set a listener in a Generator.
     */
    public static void exampleSettable(int n, int k) {
        System.out.println("One settable: n, k == " + n + ", " + k);

        final Generator g = new Generator();
        
        g.setListener(new Printer());
        g.generate(n, k);
        
        final Counter counter = new Counter();
        g.setListener(counter);
        g.generate(n, k);
        final int count = counter.getCount();
        System.out.println("# combinations = " + count);
    }
    
    /**
     * Example illustrating how to work with multiple listeners.
     */
    public static void exampleMulti(int n, int k) {
        System.out.println("Multi addable: n, k == " + n + ", " + k);

        final ObservableGenerator g = new ObservableGenerator();
        
        g.addListener(new Printer());
        final Counter counter = new Counter();
        g.addListener(counter);
        
        g.generate(n, k);
        final int count = counter.getCount();
        System.out.println("# combinations = " + count);
    }
}